package com.mortals.xhx.thread;


import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.common.Rest;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.util.ThreadPool;
import com.mortals.xhx.common.utils.AbstractThread;
import com.mortals.xhx.protocol.complex.matter.model.req.ImplementationReq;
import com.mortals.xhx.protocol.complex.matter.model.rsp.MatterListInfo;
import com.mortals.xhx.protocol.complex.matter.service.IComplexMatterService;
import com.mortals.xhx.utils.SpringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import static com.mortals.xhx.common.key.RedisKey.KEY_COMPLEX_REQ;
import static com.mortals.xhx.common.key.RedisKey.KEY_EVENT_IMPLEMENTATION_RESP;


/**
 * 发送线程
 *
 * @author zxfei
 */
@Slf4j
@Service
@Order(value = 1)
public class SendThread extends AbstractThread {

    @Autowired
    private ICacheService cacheService;

    @Value("${complex.deviceCode:''}")
    protected String deviceCode;

    @Override
    protected int getSleepTime() {
        return 100;
    }

    @Override
    protected void process() throws Exception {

        // log.info("send thread process");
        String reqStr = cacheService.lpop(KEY_COMPLEX_REQ, String.class);

        if (!ObjectUtils.isEmpty(reqStr)) {
            JSONObject reqJson = JSONObject.parseObject(reqStr);
            String urlPath = reqJson.getString("urlPath");
            String timestamp = reqJson.getString("timestamp");
            //根据url路由断言匹配发送
            if ("eventImplementation/list".equals(urlPath)) {
                //查询事项
                ImplementationReq implementationReq = JSONObject.parseObject(reqStr, ImplementationReq.class);
                ImplementListSendTask implementListSendTask = new ImplementListSendTask(deviceCode, implementationReq);
                ThreadPool.getInstance().execute(implementListSendTask);
            }


        }


    }


    @Override
    protected void threadClosed() {
        // TODO Auto-generated method stub
    }

    /**
     * 事项列表执行发送任务
     *
     * @author zxfei
     */
    class ImplementListSendTask implements Runnable {

        private ImplementationReq implementationReq;

        private String deviceLogo;

        private IComplexMatterService complexMatterService;

        private ICacheService cacheService;


        public ImplementListSendTask(String deviceLogo, ImplementationReq implementationReq) {
            this.implementationReq = implementationReq;
            this.deviceLogo = deviceLogo;
            complexMatterService = SpringUtils.getBean(IComplexMatterService.class);
            cacheService = SpringUtils.getBean(ICacheService.class);
        }

        @Override
        public void run() {
            try {
                log.info("启动发送");
                Rest<MatterListInfo> rest = complexMatterService.findMatters(deviceLogo, implementationReq);

                log.info("发送完成==>{}", JSONObject.toJSONString(rest));

                cacheService.lpushForTime(KEY_EVENT_IMPLEMENTATION_RESP + implementationReq.getNonce(),60,JSONObject.toJSONString(rest));

            } catch (Exception e) {
                log.error("发送异常：" + e);
            }


        }

    }

}
